import {fakeAsync, flush, TestBed} from '@angular/core/testing';

import { DataSourceService } from './data-source.service';

import {DataCardResponse} from '@nspop/gm-web-facade-api';
import {of, Subject} from 'rxjs';
import {HttpClientTestingModule} from '@angular/common/http/testing';
import {PatientContextService} from './patient-context.service';
import {DataCardService} from '../api/data-card.service';
import {CarePlanService} from '../api/care-plan.service';
import {ObservationService} from '../api/observation.service';

describe('DataSourceService', () => {
  let service: DataSourceService;
  let dataCardServiceStub: DataCardService;
  let carePlanServiceStub: CarePlanService;
  let observationServiceStub: ObservationService;
  let episodeOfCareStub = new Subject<string | undefined>();
  let patientContextServiceStub: PatientContextService;

  beforeEach(async () => {
    episodeOfCareStub = new Subject<string | undefined>();

    dataCardServiceStub = {
      getAll: () => of([
        {documentUniqueId: '1234', version: '5', episodeOfCareIdentifier: '1', documentStatus: 'SINGLE'} as any,
        {documentUniqueId: '789', version: '3', episodeOfCareIdentifier: '2', documentStatus: 'ITSCOMPLICATED'} as any,
      ]),
      update: (resource: any, id: string, version: string) => of({documentUniqueId: id, version: String(Number(version) + 1)} as any),
    } as any;

    carePlanServiceStub = {
      getAll: () => of(undefined),
      update: () => of(undefined),
    } as any;

    observationServiceStub = {
      getUltrasoundObservationTable: () => of(undefined),
      getClinicalObservationTable: () => of(undefined),
    } as any;

    patientContextServiceStub = {
      getCurrentEpisodeOfCare: () => '1',
      getCurrentXBreakTheGlassReason: () => undefined,
      episodeOfCare$: episodeOfCareStub,
      setHasPrivatelyMarkedData: () => {},
      getCurrentHasPrivatelyMarkedData: () => false,
    } as any;

    await TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub},
        {provide: CarePlanService, useValue: carePlanServiceStub},
        {provide: ObservationService, useValue: observationServiceStub},
        {provide: PatientContextService, useValue: patientContextServiceStub},
      ]
    }).compileComponents();
    service = TestBed.inject(DataSourceService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('updates activeDataCard$ observable after saving datacard', fakeAsync(() => {
    const invocations: Array<DataCardResponse|undefined> = [];
    service.activeDataCard$.subscribe((dataCard) => {
      invocations.push(dataCard);
    });
    service.updateDataCard({} as any, '123', '1').toPromise();
    flush();
    expect(invocations.length).toBeGreaterThanOrEqual(1);
    const card = invocations[invocations.length - 1];
    expect(card).toBeDefined();
    expect(card?.documentUniqueId).toBe('123');
    expect(card?.version).toBe('2');
  }));

  it('updates activeDataCard$ when asked to refresh', fakeAsync(() => {
    spyOn(service, 'refreshActiveDataCard').and.callThrough();
    service.refresh();
    flush();
    let called = false;
    service.activeDataCard$.subscribe((dataCard) => {
      expect(dataCard).toBeDefined();
      expect(dataCard?.documentUniqueId).toBe('1234');
      expect(dataCard?.version).toBe('5');
      called = true;
    });
    expect(service.refreshActiveDataCard).toHaveBeenCalledTimes(1);
    expect(called).toBeTrue();
  }));
});
