import {Injectable} from '@angular/core';
import {BehaviorSubject, Observable} from 'rxjs';
import {
  CarePlan,
  Activity,
  DataCard,
  DataCardResponse,
  CreateRegularObservationsRequest,
  CreateUltrasoundObservationsRequest, ObservationTable, ScanningObservationTable,
} from '@nspop/gm-web-facade-api';
import {CarePlanService} from '../api/care-plan.service';
import {DataCardService} from '../api/data-card.service';
import {tap} from 'rxjs/operators';
import {PatientContextService} from './patient-context.service';
import {ObservationService} from '../api/observation.service';

/**
 * Shared data service
 */
@Injectable({
  providedIn: 'root'
})
export class DataSourceService {
  constructor(
    private observationsService: ObservationService,
    private carePlanService: CarePlanService,
    private dataCardService: DataCardService,
    private patientContextService: PatientContextService,
  ) {}

  // Inner subjects - for manually publishing updates to subscribers
  private innerDataCardResponses: BehaviorSubject<DataCardResponse[]> = new BehaviorSubject<DataCardResponse[]>([]);
  private innerActiveDataCardResponse: BehaviorSubject<DataCardResponse | undefined>
    = new BehaviorSubject<DataCardResponse | undefined>(undefined);

  private innerCarePlan: BehaviorSubject<CarePlan | undefined> = new BehaviorSubject<CarePlan | undefined>(undefined);
  private innerActivities: BehaviorSubject<Activity[]> = new BehaviorSubject<Activity[]>([]);

  private innerUltrasoundObservationTable: BehaviorSubject<ScanningObservationTable | undefined>
    = new BehaviorSubject<ScanningObservationTable | undefined>(undefined);
  private innerClinicalObservationTable: BehaviorSubject<ObservationTable | undefined>
    = new BehaviorSubject<ObservationTable | undefined>(undefined);

  // Observables (from inner subjects) - subscribe to these to get updated info!
  public allDataCards$: Observable<DataCardResponse[]> = this.innerDataCardResponses.asObservable();
  public activeDataCard$: Observable<DataCardResponse | undefined> = this.innerActiveDataCardResponse.asObservable();

  public carePlan$: Observable<CarePlan | undefined> = this.innerCarePlan.asObservable();
  public activities$: Observable<Activity[]> = this.innerActivities.asObservable();

  public ultrasoundObservationTable$: Observable<ScanningObservationTable | undefined>
    = this.innerUltrasoundObservationTable.asObservable();
  public clinicalObservationTable$: Observable<ObservationTable | undefined>
    = this.innerClinicalObservationTable.asObservable();

  // Refresh methods - these are the ones that actually call the server and manually publishes new data
  public refreshAllDataCards(): void {
    this.dataCardService.getAll().subscribe((dataCardResponses: DataCardResponse[]) => {
      this.innerDataCardResponses.next(dataCardResponses);
      this.refreshActiveDataCard();
      this.refreshUltrasoundObservationTable();
      this.refreshClinicalObservationTable();
    });
  }

  public refreshActiveDataCard(): void {
    const dataCardResponses: DataCardResponse[] = this.innerDataCardResponses.getValue();
    const episodeOfCareId = this.patientContextService.getCurrentEpisodeOfCare();
    this.innerActiveDataCardResponse.next(dataCardResponses.find(card => card.episodeOfCareIdentifier === episodeOfCareId));
  }

  public refreshCarePlan(): void {
    const episodeOfCareId = this.patientContextService.getCurrentEpisodeOfCare();
    if (episodeOfCareId) {
      this.carePlanService.getAll().subscribe((carePlan: CarePlan) => {
        this.innerCarePlan.next(carePlan);
        this.refreshActivities();
      });
    }
    this.innerCarePlan.next(undefined);
  }

  public refreshActivities(): void {
    const carePlan: CarePlan | undefined = this.innerCarePlan.getValue();
    this.innerActivities.next(carePlan?.activities || []);
  }

  public refreshUltrasoundObservationTable(): void {
    this.observationsService.getUltrasoundObservationTable().subscribe((table: ScanningObservationTable) => {
      this.innerUltrasoundObservationTable.next(table);
    });
  }

  public refreshClinicalObservationTable(): void {
    this.observationsService.getClinicalObservationTable().subscribe((table: ObservationTable) => {
      this.innerClinicalObservationTable.next(table);
    });
  }

  public refresh(): void {
    this.refreshAllDataCards();
    this.refreshCarePlan();
  }

  // Update methods - These tap into the update server call and publishes the returned resource to subjects
  public updateDataCard(
    resource: DataCard,
    documentUniqueId: string,
    version: string,
  ): Observable<DataCardResponse> {
    return this.dataCardService.update(resource, documentUniqueId, version + '').pipe(
      tap((dataCardResponse: DataCardResponse) => {
        this.innerActiveDataCardResponse.next(dataCardResponse);
      })
    );
  }

  public updateCarePlan(
    resource: CarePlan,
    id: string,
    version: string,
  ): Observable<CarePlan> {
    return this.carePlanService.update(resource, id, version).pipe(
      tap((carePlan: CarePlan) => {
        this.innerCarePlan.next(carePlan);
      })
    );
  }

  public createUltrasoundObservations(observations: CreateUltrasoundObservationsRequest): Observable<void> {
    return this.observationsService.createUltrasoundObservations(observations);
  }

  public createRegularObservations(observations: CreateRegularObservationsRequest): Observable<void> {
    return this.observationsService.createRegularObservations(observations);
  }

  public getActivityById(id: string): Activity | null {
    const activities = this.innerActivities.getValue();
    return activities.find(activity => activity.id === id) || null;
  }
}
